@@ -1,12 +1,9 @@
+#define __INTRINSIC_ONLYSPECIAL
+#define __INTRINSIC_SPECIAL__InterlockedOr64 /* Causes code generation in intrin-impl.h */
+
 #include <intrin.h>
 
 #ifdef _WIN64
-__int64 _InterlockedOr64(__int64 volatile *Destination, __int64 Value)
-{
-  __asm__ __volatile__("lock ; orq %0,%1"
-    : : "r"(Value),"m"(*Destination) : "memory");
-  return *Destination;
-}
 #else
 __int64 __stdcall InterlockedCompareExchange64(__int64 volatile *Destination,
   __int64 Exchange, __int64 Comperand);
@@ -24,6 +21,7 @@ __int64 _InterlockedOr64 (__int64 volatile *Destination,__int64 Value)
 #ifdef _WIN64
 __int64 InterlockedOr64(__int64 volatile *, __int64) __attribute__((alias("_InterlockedOr64")));
 #else
+__int64 __stdcall InterlockedOr64(__int64 volatile *Destination, __int64 Value);
 __int64 __stdcall InterlockedOr64(__int64 volatile *Destination, __int64 Value)
 {
   return _InterlockedOr64(Destination, Value);
