@@ -4,8 +4,6 @@
 #include <errno.h>
 #include <msvcrt.h>
 
-errno_t __cdecl asctime_s (char *, size_t, const struct tm *);
-/* char * __cdecl asctime (const struct tm *); */
 static errno_t __cdecl _int_asctime_s (char *, size_t, const struct tm *);
 static errno_t __cdecl _stub (char *, size_t, const struct tm *);
 
