@@ -564,8 +564,21 @@ interface ID2D1Bitmap1 : public ID2D1Bitmap
 #else
 
 typedef interface ID2D1Bitmap1 ID2D1Bitmap1;
-/* FIXME: Add full C declaration */
 
+typedef struct ID2D1Bitmap1Vtbl {
+    ID2D1BitmapVtbl Base;
+
+    STDMETHOD_(void, GetColorContext)(ID2D1Bitmap1 *This, ID2D1ColorContext **colorContext) PURE;
+    STDMETHOD_(D2D1_BITMAP_OPTIONS, GetOptions)(ID2D1Bitmap1 *This) PURE;
+    STDMETHOD(GetSurface)(ID2D1Bitmap1 *This, IDXGISurface **dxgiSurface) PURE;
+    STDMETHOD(Map)(ID2D1Bitmap1 *This, D2D1_MAP_OPTIONS options, D2D1_MAPPED_RECT *mappedRect) PURE;
+    STDMETHOD(Unmap)(ID2D1Bitmap1 *This) PURE;
+}
+ID2D1Bitmap1Vtbl;
+
+interface ID2D1Bitmap1 {
+    const ID2D1Bitmap1Vtbl *lpVtbl;
+};
 #endif
 
 DEFINE_GUID(IID_ID2D1Bitmap1, 0xa898a84c,0x3873,0x4588,0xb0,0x8b,0xeb,0xbf,0x97,0x8d,0xf0,0x41);
@@ -864,7 +877,130 @@ interface ID2D1DeviceContext : public ID2D1RenderTarget
 #else
 
 typedef interface ID2D1DeviceContext ID2D1DeviceContext;
-/* FIXME: Add full C declaration */
+
+typedef struct ID2D1DeviceContextVtbl {
+    struct ID2D1RenderTargetVtbl Base;
+
+    STDMETHOD(CreateBitmap)(ID2D1DeviceContext *This, D2D1_SIZE_U size,
+        CONST void *sourceData, UINT32 pitch,
+        CONST D2D1_BITMAP_PROPERTIES1 *bitmapProperties,
+        ID2D1Bitmap1 **bitmap) PURE;
+
+    STDMETHOD(CreateBitmapFromWicBitmap)(ID2D1DeviceContext *This,
+        IWICBitmapSource *wicBitmapSource,
+        CONST D2D1_BITMAP_PROPERTIES1 *bitmapProperties,
+        ID2D1Bitmap1 **bitmap) PURE;
+
+    STDMETHOD(CreateColorContext)(ID2D1DeviceContext *This,
+        D2D1_COLOR_SPACE space, CONST BYTE *profile, UINT32 profileSize,
+        ID2D1ColorContext **colorContext) PURE;
+    STDMETHOD(CreateColorContextFromFilename)(ID2D1DeviceContext *This,
+        PCWSTR filename, ID2D1ColorContext **colorContext) PURE;
+    STDMETHOD(CreateColorContextFromWicColorContext)(ID2D1DeviceContext *This,
+        IWICColorContext *wicColorContext,
+        ID2D1ColorContext **colorContext) PURE;
+    STDMETHOD(CreateBitmapFromDxgiSurface)(ID2D1DeviceContext *This,
+        IDXGISurface *surface, CONST D2D1_BITMAP_PROPERTIES1 *bitmapProperties,
+        ID2D1Bitmap1 **bitmap) PURE;
+    STDMETHOD(CreateEffect)(ID2D1DeviceContext *This, REFCLSID effectId,
+        ID2D1Effect **effect) PURE;
+
+    STDMETHOD(CreateGradientStopCollection)(ID2D1DeviceContext *This,
+        CONST D2D1_GRADIENT_STOP *straightAlphaGradientStops,
+        UINT32 straightAlphaGradientStopsCount,
+        D2D1_COLOR_SPACE preInterpolationSpace,
+        D2D1_COLOR_SPACE postInterpolationSpace,
+        D2D1_BUFFER_PRECISION bufferPrecision,
+        D2D1_EXTEND_MODE extendMode,
+        D2D1_COLOR_INTERPOLATION_MODE colorInterpolationMode,
+        ID2D1GradientStopCollection1 **gradientStopCollection1) PURE;
+
+    STDMETHOD(CreateImageBrush)(ID2D1DeviceContext *This,
+        struct ID2D1Image *image,
+        CONST D2D1_IMAGE_BRUSH_PROPERTIES *imageBrushProperties,
+        CONST D2D1_BRUSH_PROPERTIES *brushProperties,
+        ID2D1ImageBrush **imageBrush) PURE;
+
+    STDMETHOD(CreateBitmapBrush)(ID2D1DeviceContext *This, ID2D1Bitmap *bitmap,
+        CONST D2D1_BITMAP_BRUSH_PROPERTIES1 *bitmapBrushProperties,
+        CONST D2D1_BRUSH_PROPERTIES *brushProperties,
+        ID2D1BitmapBrush1 **bitmapBrush) PURE;
+
+    STDMETHOD(CreateCommandList)(ID2D1DeviceContext *This,
+        ID2D1CommandList **commandList) PURE;
+    STDMETHOD_(BOOL, IsDxgiFormatSupported)(ID2D1DeviceContext *This,
+        DXGI_FORMAT format) PURE;
+    STDMETHOD_(BOOL, IsBufferPrecisionSupported)(ID2D1DeviceContext *This,
+        D2D1_BUFFER_PRECISION bufferPrecision) PURE;
+    STDMETHOD(GetImageLocalBounds)(ID2D1DeviceContext *This,
+        struct ID2D1Image *image, D2D1_RECT_F *localBounds) PURE;
+    STDMETHOD(GetImageWorldBounds)(ID2D1DeviceContext *This,
+        struct ID2D1Image *image, D2D1_RECT_F *worldBounds) PURE;
+    STDMETHOD(GetGlyphRunWorldBounds)(ID2D1DeviceContext *This,
+        D2D1_POINT_2F baselineOrigin, CONST DWRITE_GLYPH_RUN *glyphRun,
+        DWRITE_MEASURING_MODE measuringMode, D2D1_RECT_F *bounds) PURE;
+    STDMETHOD_(void, GetDevice)(ID2D1DeviceContext *This,
+        ID2D1Device **device) PURE;
+    STDMETHOD_(void, SetTarget)(ID2D1DeviceContext *This,
+        struct ID2D1Image *image) PURE;
+    STDMETHOD_(void, GetTarget)(ID2D1DeviceContext *This,
+        struct ID2D1Image **image) PURE;
+    STDMETHOD_(void, SetRenderingControls)(ID2D1DeviceContext *This,
+        CONST D2D1_RENDERING_CONTROLS *renderingControls) PURE;
+    STDMETHOD_(void, GetRenderingControls)(ID2D1DeviceContext *This,
+        D2D1_RENDERING_CONTROLS *renderingControls) PURE;
+    STDMETHOD_(void, SetPrimitiveBlend)(ID2D1DeviceContext *This,
+        D2D1_PRIMITIVE_BLEND primitiveBlend) PURE;
+    STDMETHOD_(D2D1_PRIMITIVE_BLEND, GetPrimitiveBlend)(ID2D1DeviceContext *This) PURE;
+    STDMETHOD_(void, SetUnitMode)(ID2D1DeviceContext *This,
+        D2D1_UNIT_MODE unitMode) PURE;
+    STDMETHOD_(D2D1_UNIT_MODE, GetUnitMode)(ID2D1DeviceContext *This) PURE;
+
+    STDMETHOD_(void, DrawGlyphRun)(ID2D1DeviceContext *This,
+        D2D1_POINT_2F baselineOrigin, CONST DWRITE_GLYPH_RUN *glyphRun,
+        CONST DWRITE_GLYPH_RUN_DESCRIPTION *glyphRunDescription,
+        ID2D1Brush *foregroundBrush, DWRITE_MEASURING_MODE measuringMode) PURE;
+
+    STDMETHOD_(void, DrawImage)(ID2D1DeviceContext *This,
+        struct ID2D1Image *image, CONST D2D1_POINT_2F *targetOffset,
+        CONST D2D1_RECT_F *imageRectangle,
+        D2D1_INTERPOLATION_MODE interpolationMode,
+        D2D1_COMPOSITE_MODE compositeMode) PURE;
+    STDMETHOD_(void, DrawGdiMetafile)(ID2D1DeviceContext *This,
+        ID2D1GdiMetafile *gdiMetafile, CONST D2D1_POINT_2F *targetOffset) PURE;
+
+    STDMETHOD_(void, DrawBitmap)(ID2D1DeviceContext *This,
+        ID2D1Bitmap *bitmap, CONST D2D1_RECT_F *destinationRectangle,
+        FLOAT opacity, D2D1_INTERPOLATION_MODE interpolationMode,
+        CONST D2D1_RECT_F *sourceRectangle,
+        CONST D2D1_MATRIX_4X4_F *perspectiveTransform) PURE;
+
+    STDMETHOD_(void, PushLayer)(ID2D1DeviceContext *This,
+        CONST D2D1_LAYER_PARAMETERS1 *layerParameters, ID2D1Layer *layer) PURE;
+
+    STDMETHOD(InvalidateEffectInputRectangle)(ID2D1DeviceContext *This,
+        ID2D1Effect *effect, UINT32 input,
+        CONST D2D1_RECT_F *inputRectangle) PURE;
+    STDMETHOD(GetEffectInvalidRectangleCount)(ID2D1DeviceContext *This,
+        ID2D1Effect *effect, UINT32 *rectangleCount) PURE;
+    STDMETHOD(GetEffectInvalidRectangles)(ID2D1DeviceContext *This,
+        ID2D1Effect *effect, D2D1_RECT_F *rectangles,
+        UINT32 rectanglesCount) PURE;
+    STDMETHOD(GetEffectRequiredInputRectangles)(ID2D1DeviceContext *This,
+        ID2D1Effect *renderEffect, CONST D2D1_RECT_F *renderImageRectangle,
+        CONST D2D1_EFFECT_INPUT_DESCRIPTION *inputDescriptions,
+        D2D1_RECT_F *requiredInputRects, UINT32 inputCount) PURE;
+
+    STDMETHOD_(void, FillOpacityMask)(ID2D1DeviceContext *This,
+        ID2D1Bitmap *opacityMask, ID2D1Brush *brush,
+        CONST D2D1_RECT_F *destinationRectangle,
+        CONST D2D1_RECT_F *sourceRectangle) PURE;
+}
+ID2D1DeviceContextVtbl;
+
+interface ID2D1DeviceContext {
+    const ID2D1DeviceContextVtbl *lpVtbl;
+};
 
 #endif
 
@@ -891,7 +1027,29 @@ interface ID2D1Device : public ID2D1Resource
 #else
 
 typedef interface ID2D1Device ID2D1Device;
-/* FIXME: Add full C declaration */
+
+typedef struct ID2D1DeviceVtbl {
+    struct ID2D1ResourceVtbl Base;
+
+    STDMETHOD(CreateDeviceContext)(ID2D1Device *This,
+        D2D1_DEVICE_CONTEXT_OPTIONS options,
+        ID2D1DeviceContext **deviceContext) PURE;
+    STDMETHOD(CreatePrintControl)(ID2D1Device *This,
+        IWICImagingFactory *wicFactory,
+        IPrintDocumentPackageTarget *documentTarget,
+        CONST D2D1_PRINT_CONTROL_PROPERTIES *printControlProperties,
+        ID2D1PrintControl **printControl) PURE;
+    STDMETHOD_(void, SetMaximumTextureMemory)(ID2D1Device *This,
+        UINT64 maximumInBytes) PURE;
+    STDMETHOD_(UINT64, GetMaximumTextureMemory)(ID2D1Device *This) PURE;
+    STDMETHOD_(void, ClearResources)(ID2D1Device *This,
+        UINT32 millisecondsSinceUse) PURE;
+}
+ID2D1DeviceVtbl;
+
+interface ID2D1Device {
+    const ID2D1DeviceVtbl *lpVtbl;
+};
 
 #endif
 
@@ -999,7 +1157,57 @@ interface ID2D1Factory1 : public ID2D1Factory
 #else
 
 typedef interface ID2D1Factory1 ID2D1Factory1;
-/* FIXME: Add full C declaration */
+
+typedef struct ID2D1Factory1Vtbl {
+    ID2D1FactoryVtbl Base;
+
+    STDMETHOD(CreateDevice)(ID2D1Factory1 *This, IDXGIDevice *dxgiDevice,
+            ID2D1Device **d2dDevice) PURE;
+    STDMETHOD(CreateStrokeStyle)(ID2D1Factory1 *This,
+            CONST D2D1_STROKE_STYLE_PROPERTIES1 *strokeStyleProperties,
+            CONST FLOAT *dashes, UINT32 dashesCount,
+            ID2D1StrokeStyle1 **strokeStyle) PURE;
+    STDMETHOD(CreatePathGeometry)(ID2D1Factory1 *This,
+            ID2D1PathGeometry1 **pathGeometry) PURE;
+    STDMETHOD(CreateDrawingStateBlock)(ID2D1Factory1 *This,
+            CONST D2D1_DRAWING_STATE_DESCRIPTION1 *drawingStateDescription,
+            IDWriteRenderingParams *textRenderingParams,
+            ID2D1DrawingStateBlock1 **drawingStateBlock) PURE;
+    STDMETHOD(CreateGdiMetafile)(ID2D1Factory1 *This, IStream *metafileStream,
+            ID2D1GdiMetafile **metafile) PURE;
+    STDMETHOD(RegisterEffectFromStream)(ID2D1Factory1 *This, REFCLSID classId,
+            IStream *propertyXml, CONST D2D1_PROPERTY_BINDING *bindings,
+            UINT32 bindingsCount,
+            CONST PD2D1_EFFECT_FACTORY effectFactory) PURE;
+    STDMETHOD(RegisterEffectFromString)(ID2D1Factory1 *This,
+            REFCLSID classId, PCWSTR propertyXml,
+            CONST D2D1_PROPERTY_BINDING *bindings, UINT32 bindingsCount,
+            CONST PD2D1_EFFECT_FACTORY effectFactory) PURE;
+    STDMETHOD(UnregisterEffect)(ID2D1Factory1 *This, REFCLSID classId) PURE;
+    STDMETHOD(GetRegisteredEffects)(ID2D1Factory1 *This, CLSID *effects,
+            UINT32 effectsCount, UINT32 *effectsReturned,
+            UINT32 *effectsRegistered) PURE;
+    STDMETHOD(GetEffectProperties)(ID2D1Factory1 *This, REFCLSID effectId,
+            ID2D1Properties **properties) PURE;
+} ID2D1Factory1Vtbl;
+
+interface ID2D1Factory1 {
+    const ID2D1Factory1Vtbl *lpVtbl;
+};
+
+#define ID2D1Factory1_QueryInterface(this,A,B) (this)->lpVtbl->Base.Base.QueryInterface((IUnknown *)(this),A,B)
+#define ID2D1Factory1_AddRef(this) (this)->lpVtbl->Base.Base.AddRef((IUnknown *)(this))
+#define ID2D1Factory1_Release(this) (this)->lpVtbl->Base.Base.Release((IUnknown *)(this))
+#define ID2D1Factory1_CreateDevice(this,A,B) (this)->lpVtbl->CreateDevice(this,A,B)
+#define ID2D1Factory1_CreateStrokeStyle(this,A,B,C,D) (this)->lpVtbl->CreateStrokeStyle(this,A,B,C,D)
+#define ID2D1Factory1_CreatePathGeometry(this,A) (this)->lpVbtl->CreatePathGeometry(this,A)
+#define ID2D1Factory1_CreateDrawingStateBlock(this,A,B, C) (this)->lpVtbl->CreateDrawingStateBlock(this,A,B,C)
+#define ID2D1Factory1_CreateGdiMetafile(this,A,B) (this)->lpVtbl->CreateGdiMetafile(this,A,B)
+#define ID2D1Factory1_RegisterEffectFromStream(this,A,B,C,D,E) (this)->lpVtbl->RegisterEffectFromStream(this,A,B,C,D,E)
+#define ID2D1Factory1_RegisterEffectFromString(this,A,B,C,D,E) (this)->lpVtbl->RegisterEffectFromString(this,A,B,C,D,E)
+#define ID2D1Factory1_UnregisterEffect(this,A) (this)->lpVtbl->UnregisterEffect(this,A)
+#define ID2D1Factory1_GetRegisteredEffects(this,A,B,C,D) (this)->lpVtbl->GetRegisteredEffects(this,A,B,C,D)
+#define ID2D1Factory1_GetEffectProperties(this,A,B) (this)->lpVtbl->GetEffectProperties(this,A,B)
 
 #endif
 
@@ -1018,7 +1226,25 @@ interface ID2D1Multithread : public IUnknown
 #else
 
 typedef interface ID2D1Multithread ID2D1Multithread;
-/* FIXME: Add full C declaration */
+
+typedef struct ID2D1MultithreadVtbl {
+    IUnknownVtbl Base;
+
+    STDMETHOD_(BOOL, GetMultithreadProtected)(ID2D1Multithread *This) PURE;
+    STDMETHOD_(void, Enter)(ID2D1Multithread *This) PURE;
+    STDMETHOD_(void, Leave)(ID2D1Multithread *This) PURE;
+} ID2D1MultithreadVtbl;
+
+interface ID2D1Multithread {
+    ID2D1MultithreadVtbl *lpVtbl;
+};
+
+#define ID2D1Multithread_QueryInterface(this,A,B) (this)->lpVtbl->Base.QueryInterface((IUnknown *)(this),A,B)
+#define ID2D1Multithread_AddRef(this) (this)->lpVtbl->Base.AddRef((IUnknown *)(this))
+#define ID2D1Multithread_Release(this) (this)->lpVtbl->Base.Release((IUnknown *)(this))
+#define ID2D1Mutlithread_GetMultithreadProtected(this) (this)->lpVtbl->GetMultihreadProtected(this)
+#define ID2D1Multithread_Enter(this) (this)->lpVtbl->Enter(this)
+#define ID2D1Multithread_Leave(this) (this)->lpVtbl->Leave(this)
 
 #endif
 
