@@ -181,7 +181,7 @@ extern "C" {
 #define NCRYPT_REGISTER_NOTIFY_FLAG 0x1
 #define NCRYPT_UNREGISTER_NOTIFY_FLAG 0x2
 
-#if WINAPI_FAMILY_PARTITION (WINAPI_PARTITION_DESKTOP)
+#if WINAPI_FAMILY_PARTITION (WINAPI_PARTITION_DESKTOP) || _WIN32_WINNT >= 0x0A00
 #define NCRYPT_CIPHER_OPERATION BCRYPT_CIPHER_OPERATION
 #define NCRYPT_HASH_OPERATION BCRYPT_HASH_OPERATION
 #define NCRYPT_ASYMMETRIC_ENCRYPTION_OPERATION BCRYPT_ASYMMETRIC_ENCRYPTION_OPERATION
