@@ -1,6 +1,6 @@
 /**
  * This file has no copyright assigned and is placed in the Public Domain.
- * This file is part of the w64 mingw-runtime package.
+ * This file is part of the mingw-w64 runtime package.
  * No warranty is given; refer to the file DISCLAIMER.PD within this package.
  */
 #ifndef _WINCON_
@@ -8,10 +8,15 @@
 
 #include <_mingw_unicode.h>
 
+#include <winapifamily.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+#define CONSOLE_REAL_OUTPUT_HANDLE (LongToHandle(-2))
+#define CONSOLE_REAL_INPUT_HANDLE (LongToHandle(-3))
+
   typedef struct _COORD {
     SHORT X;
     SHORT Y;
@@ -51,6 +56,7 @@ extern "C" {
 #define NLS_HIRAGANA 0x40000
 #define NLS_ROMAN 0x400000
 #define NLS_IME_CONVERSION 0x800000
+#define ALTNUMPAD_BIT 0x4000000
 #define NLS_IME_DISABLE 0x20000000
 
   typedef struct _MOUSE_EVENT_RECORD {
@@ -69,6 +75,9 @@ extern "C" {
 #define MOUSE_MOVED 0x1
 #define DOUBLE_CLICK 0x2
 #define MOUSE_WHEELED 0x4
+#if (_WIN32_WINNT >= 0x0600)
+#define MOUSE_HWHEELED 0x8
+#endif
 
   typedef struct _WINDOW_BUFFER_SIZE_RECORD {
     COORD dwSize;
@@ -173,9 +182,15 @@ extern "C" {
 #define ENABLE_QUICK_EDIT_MODE 0x40
 #define ENABLE_EXTENDED_FLAGS 0x80
 #define ENABLE_AUTO_POSITION 0x100
+#define ENABLE_VIRTUAL_TERMINAL_INPUT 0x200
 
 #define ENABLE_PROCESSED_OUTPUT 0x1
 #define ENABLE_WRAP_AT_EOL_OUTPUT 0x2
+#define ENABLE_VIRTUAL_TERMINAL_PROCESSING 0x4
+#define DISABLE_NEWLINE_AUTO_RETURN 0x8
+#define ENABLE_LVB_GRID_WORLDWIDE 0x10
+
+#if WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_APP)
 
 #define PeekConsoleInput __MINGW_NAME_AW(PeekConsoleInput)
 #define ReadConsoleInput __MINGW_NAME_AW(ReadConsoleInput)
@@ -262,8 +277,12 @@ extern "C" {
 
 #define CONSOLE_FULLSCREEN 1
 #define CONSOLE_FULLSCREEN_HARDWARE 2
-
   WINBASEAPI WINBOOL WINAPI GetConsoleDisplayMode(LPDWORD lpModeFlags);
+
+#define CONSOLE_FULLSCREEN_MODE 1
+#define CONSOLE_WINDOWED_MODE 2
+  WINBASEAPI WINBOOL WINAPI SetConsoleDisplayMode(HANDLE hConsoleOutput, DWORD dwFlags, PCOORD lpNewScreenBufferDimensions);
+
   WINBASEAPI HWND WINAPI GetConsoleWindow(VOID);
   WINBASEAPI DWORD WINAPI GetConsoleProcessList(LPDWORD lpdwProcessList,DWORD dwProcessCount);
   WINBASEAPI WINBOOL WINAPI AddConsoleAliasA(LPSTR Source,LPSTR Target,LPSTR ExeName);
@@ -279,7 +298,22 @@ extern "C" {
   WINBASEAPI DWORD WINAPI GetConsoleAliasExesA(LPSTR ExeNameBuffer,DWORD ExeNameBufferLength);
   WINBASEAPI DWORD WINAPI GetConsoleAliasExesW(LPWSTR ExeNameBuffer,DWORD ExeNameBufferLength);
 
-#if (_WIN32_WINNT >= 0x0600)
+WINBASEAPI VOID WINAPI ExpungeConsoleCommandHistoryA(LPSTR ExeName);
+WINBASEAPI VOID WINAPI ExpungeConsoleCommandHistoryW(LPWSTR ExeName);
+#define ExpungeConsoleCommandHistory __MINGW_NAME_AW(ExpungeConsoleCommandHistory)
+
+WINBASEAPI WINBOOL WINAPI SetConsoleNumberOfCommandsA(DWORD Number, LPSTR ExeName);
+WINBASEAPI WINBOOL WINAPI SetConsoleNumberOfCommandsW(DWORD Number, LPWSTR ExeName);
+#define SetConsoleNumberOfCommands __MINGW_NAME_AW(SetConsoleNumberOfCommands)
+
+WINBASEAPI DWORD WINAPI GetConsoleCommandHistoryLengthA(LPSTR ExeName);
+WINBASEAPI DWORD WINAPI GetConsoleCommandHistoryLengthW(LPWSTR ExeName);
+#define GetConsoleCommandHistoryLength __MINGW_NAME_AW(GetConsoleCommandHistoryLength)
+
+WINBASEAPI DWORD WINAPI GetConsoleCommandHistoryA(LPSTR Commands, DWORD CommandBufferLength, LPSTR ExeName);
+WINBASEAPI DWORD WINAPI GetConsoleCommandHistoryW(LPWSTR Commands, DWORD CommandBufferLength, LPWSTR ExeName);
+#define GetConsoleCommandHistory __MINGW_NAME_AW(GetConsoleCommandHistory)
+
 #ifndef LF_FACESIZE
 #define LF_FACESIZE 32
 #endif
@@ -293,6 +327,8 @@ typedef struct _CONSOLE_FONT_INFOEX {
   WCHAR FaceName[LF_FACESIZE];
 } CONSOLE_FONT_INFOEX, *PCONSOLE_FONT_INFOEX;
 
+#define HISTORY_NO_DUP_FLAG 0x1
+
 typedef struct _CONSOLE_HISTORY_INFO {
   UINT  cbSize;
   UINT  HistoryBufferSize;
@@ -315,7 +351,7 @@ typedef struct _CONSOLE_SCREEN_BUFFER_INFOEX {
   SMALL_RECT srWindow;
   COORD      dwMaximumWindowSize;
   WORD       wPopupAttributes;
-  BOOL       bFullscreenSupported;
+  WINBOOL    bFullscreenSupported;
   COLORREF   ColorTable[16];
 } CONSOLE_SCREEN_BUFFER_INFOEX, *PCONSOLE_SCREEN_BUFFER_INFOEX;
 
@@ -323,6 +359,7 @@ WINBOOL WINAPI GetConsoleHistoryInfo(
   PCONSOLE_HISTORY_INFO lpConsoleHistoryInfo
 );
 
+#if (_WIN32_WINNT >= 0x0600)
 #define GetConsoleOriginalTitle __MINGW_NAME_AW(GetConsoleOriginalTitle)
 
 WINBASEAPI DWORD WINAPI GetConsoleOriginalTitleA(
@@ -334,6 +371,7 @@ WINBASEAPI DWORD WINAPI GetConsoleOriginalTitleW(
   LPWSTR lpConsoleTitle,
   DWORD nSize
 );
+#endif /* (_WIN32_WINNT >= 0x0600) */
 
 WINBASEAPI WINBOOL WINAPI GetConsoleScreenBufferInfoEx(
   HANDLE hConsoleOutput,
@@ -361,7 +399,24 @@ WINBASEAPI WINBOOL WINAPI SetCurrentConsoleFontEx(
   PCONSOLE_FONT_INFOEX lpConsoleCurrentFontEx
 );
 
-#endif /* (_WIN32_WINNT >= 0x0600) */
+#endif /* WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_APP) */
+
+#if WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
+
+#if (NTDDI_VERSION >= 0x0A000006)
+
+/* CreatePseudoConsole Flags */
+#define PSEUDOCONSOLE_INHERIT_CURSOR 1
+
+typedef VOID* HPCON;
+
+WINBASEAPI HRESULT WINAPI CreatePseudoConsole(COORD size, HANDLE hInput, HANDLE hOutput, DWORD dwFlags, HPCON* phPC);
+WINBASEAPI HRESULT WINAPI ResizePseudoConsole(HPCON hPC, COORD size);
+WINBASEAPI VOID WINAPI ClosePseudoConsole(HPCON hPC);
+
+#endif /* NTDDI_WIN10_RS5 */
+
+#endif /* WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP) */
 
 #ifdef __cplusplus
 }
