@@ -24,7 +24,7 @@
 #include <stdarg.h>
 #include <assert.h>
 #include "guiddef.h"
-#include "wine/rpcfc.h"
+#include "ndrtypes.h"
 #include "wine/list.h"
 
 #ifndef UUID_DEFINED
@@ -40,6 +40,7 @@ typedef struct _attr_t attr_t;
 typedef struct _expr_t expr_t;
 typedef struct _type_t type_t;
 typedef struct _var_t var_t;
+typedef struct _decl_spec_t decl_spec_t;
 typedef struct _declarator_t declarator_t;
 typedef struct _ifref_t ifref_t;
 typedef struct _typelib_entry_t typelib_entry_t;
@@ -51,6 +52,7 @@ typedef struct _user_type_t context_handle_t;
 typedef struct _user_type_t generic_handle_t;
 typedef struct _type_list_t type_list_t;
 typedef struct _statement_t statement_t;
+typedef struct _warning_t warning_t;
 
 typedef struct list attr_list_t;
 typedef struct list str_list_t;
@@ -58,11 +60,11 @@ typedef struct list expr_list_t;
 typedef struct list var_list_t;
 typedef struct list declarator_list_t;
 typedef struct list ifref_list_t;
-typedef struct list array_dims_t;
 typedef struct list user_type_list_t;
 typedef struct list context_handle_list_t;
 typedef struct list generic_handle_list_t;
 typedef struct list statement_list_t;
+typedef struct list warning_list_t;
 
 enum attr_type
 {
@@ -79,7 +81,6 @@ enum attr_type
     ATTR_CASE,
     ATTR_CODE,
     ATTR_COMMSTATUS,
-    ATTR_CONST, /* const pseudo-attribute */
     ATTR_CONTEXTHANDLE,
     ATTR_CONTROL,
     ATTR_DECODE,
@@ -114,7 +115,6 @@ enum attr_type
     ATTR_IMMEDIATEBIND,
     ATTR_IMPLICIT_HANDLE,
     ATTR_IN,
-    ATTR_INLINE,
     ATTR_INPUTSYNC,
     ATTR_LENGTHIS,
     ATTR_LIBLCID,
@@ -233,6 +233,16 @@ enum storage_class
     STG_REGISTER,
 };
 
+enum type_qualifier
+{
+    TYPE_QUALIFIER_CONST = 1,
+};
+
+enum function_specifier
+{
+    FUNCTION_SPECIFIER_INLINE = 1,
+};
+
 enum statement_type
 {
     STMT_LIBRARY,
@@ -264,6 +274,7 @@ enum type_basic_type
     TYPE_BASIC_INT64,
     TYPE_BASIC_INT,
     TYPE_BASIC_INT3264,
+    TYPE_BASIC_LONG,
     TYPE_BASIC_CHAR,
     TYPE_BASIC_HYPER,
     TYPE_BASIC_BYTE,
@@ -291,6 +302,14 @@ struct str_list_entry_t
     struct list entry;
 };
 
+struct _decl_spec_t
+{
+  type_t *type;
+  enum storage_class stgclass;
+  enum type_qualifier qualifier;
+  enum function_specifier func_specifier;
+};
+
 struct _attr_t {
   enum attr_type type;
   union {
@@ -309,7 +328,7 @@ struct _expr_t {
     double dval;
     const char *sval;
     const expr_t *ext;
-    type_t *tref;
+    decl_spec_t tref;
   } u;
   const expr_t *ext2;
   int is_const;
@@ -332,7 +351,6 @@ struct func_details
 {
   var_list_t *args;
   struct _var_t *retval;
-  int idx;
 };
 
 struct iface_details
@@ -341,6 +359,8 @@ struct iface_details
   var_list_t *disp_methods;
   var_list_t *disp_props;
   struct _type_t *inherit;
+  struct _type_t *disp_inherit;
+  struct _type_t *async_iface;
 };
 
 struct module_details
@@ -352,9 +372,8 @@ struct array_details
 {
   expr_t *size_is;
   expr_t *length_is;
-  struct _type_t *elem;
+  struct _decl_spec_t elem;
   unsigned int dim;
-  unsigned char ptr_def_fc;
   unsigned char declptr; /* if declared as a pointer */
   unsigned short ptr_tfsoff;  /* offset of pointer definition for declptr */
 };
@@ -372,8 +391,7 @@ struct basic_details
 
 struct pointer_details
 {
-  struct _type_t *ref;
-  unsigned char def_fc;
+  struct _decl_spec_t ref;
 };
 
 struct bitfield_details
@@ -382,6 +400,21 @@ struct bitfield_details
   const expr_t *bits;
 };
 
+struct alias_details
+{
+    struct _decl_spec_t aliasee;
+};
+
+#define HASHMAX 64
+
+struct namespace {
+    const char *name;
+    struct namespace *parent;
+    struct list entry;
+    struct list children;
+    struct rtype *type_hash[HASHMAX];
+};
+
 enum type_type
 {
     TYPE_VOID,
@@ -402,6 +435,7 @@ enum type_type
 
 struct _type_t {
   const char *name;
+  struct namespace *namespace;
   enum type_type type_type;
   attr_list_t *attrs;
   union
@@ -416,8 +450,9 @@ struct _type_t {
     struct basic_details basic;
     struct pointer_details pointer;
     struct bitfield_details bitfield;
+    struct alias_details alias;
   } details;
-  type_t *orig;                   /* dup'd types */
+  const char *c_name;
   unsigned int typestring_offset;
   unsigned int ptrdesc;           /* used for complex structs */
   int typelib_idx;
@@ -428,20 +463,23 @@ struct _type_t {
   unsigned int user_types_registered : 1;
   unsigned int tfswrite : 1;   /* if the type needs to be written to the TFS */
   unsigned int checked : 1;
-  unsigned int is_alias : 1; /* is the type an alias? */
 };
 
 struct _var_t {
   char *name;
-  type_t *type;
+  decl_spec_t declspec;
   attr_list_t *attrs;
   expr_t *eval;
-  enum storage_class stgclass;
-  unsigned int procstring_offset;
+
   unsigned int typestring_offset;
 
+  /* fields specific to functions */
+  unsigned int procstring_offset, func_idx;
+
   struct _loc_info_t loc_info;
 
+  unsigned int declonly : 1;
+
   /* parser-internal */
   struct list entry;
 };
@@ -449,8 +487,7 @@ struct _var_t {
 struct _declarator_t {
   var_t *var;
   type_t *type;
-  type_t *func_type;
-  array_dims_t *array;
+  enum type_qualifier qualifier;
   expr_t *bits;
 
   /* parser-internal */
@@ -482,6 +519,7 @@ struct _importinfo_t {
 };
 
 struct _importlib_t {
+    int offset;
     char *name;
 
     int version;
@@ -500,6 +538,9 @@ struct _typelib_t {
     const attr_list_t *attrs;
     struct list importlibs;
     statement_list_t *stmts;
+
+    type_t **reg_ifaces;
+    unsigned int reg_iface_count;
 };
 
 struct _user_type_t {
@@ -524,6 +565,12 @@ struct _statement_t {
         typelib_t *lib;
         type_list_t *type_list;
     } u;
+    unsigned int declonly : 1; /* for STMT_TYPE and STMT_TYPEDEF */
+};
+
+struct _warning_t {
+    int num;
+    struct list entry;
 };
 
 typedef enum {
@@ -533,11 +580,10 @@ typedef enum {
     SYS_WIN64
 } syskind_t;
 
-extern syskind_t typelib_kind;
 extern user_type_list_t user_type_list;
 extern context_handle_list_t context_handle_list;
 extern generic_handle_list_t generic_handle_list;
-void check_for_additional_prototype_types(const var_list_t *list);
+void check_for_additional_prototype_types(type_t *type);
 
 void init_types(void);
 type_t *alloc_type(void);
@@ -549,33 +595,27 @@ void clear_all_offsets(void);
 #define tsUNION  3
 
 var_t *find_const(const char *name, int f);
-type_t *find_type(const char *name, int t);
+type_t *find_type(const char *name, struct namespace *namespace, int t);
 type_t *make_type(enum type_type type);
-type_t *get_type(enum type_type type, char *name, int t);
-type_t *reg_type(type_t *type, const char *name, int t);
-void add_incomplete(type_t *t);
+type_t *get_type(enum type_type type, char *name, struct namespace *namespace, int t);
+type_t *reg_type(type_t *type, const char *name, struct namespace *namespace, int t);
 
 var_t *make_var(char *name);
 var_list_t *append_var(var_list_t *list, var_t *var);
 
 void init_loc_info(loc_info_t *);
 
-static inline var_list_t *type_get_function_args(const type_t *func_type)
-{
-  return func_type->details.function->args;
-}
+char *format_namespace(struct namespace *namespace, const char *prefix, const char *separator, const char *suffix);
 
 static inline enum type_type type_get_type_detect_alias(const type_t *type)
 {
-    if (type->is_alias)
-        return TYPE_ALIAS;
     return type->type_type;
 }
 
 #define STATEMENTS_FOR_EACH_FUNC(stmt, stmts) \
   if (stmts) LIST_FOR_EACH_ENTRY( stmt, stmts, statement_t, entry ) \
-    if (stmt->type == STMT_DECLARATION && stmt->u.var->stgclass == STG_NONE && \
-        type_get_type_detect_alias(stmt->u.var->type) == TYPE_FUNCTION)
+    if (stmt->type == STMT_DECLARATION && stmt->u.var->declspec.stgclass == STG_NONE && \
+        type_get_type_detect_alias(stmt->u.var->declspec.type) == TYPE_FUNCTION)
 
 static inline int statements_has_func(const statement_list_t *stmts)
 {
@@ -589,4 +629,19 @@ static inline int statements_has_func(const statement_list_t *stmts)
   return has_func;
 }
 
+static inline int is_global_namespace(const struct namespace *namespace)
+{
+    return !namespace->name;
+}
+
+static inline decl_spec_t *init_declspec(decl_spec_t *declspec, type_t *type)
+{
+  declspec->type = type;
+  declspec->stgclass = STG_NONE;
+  declspec->qualifier = 0;
+  declspec->func_specifier = 0;
+
+  return declspec;
+}
+
 #endif
